/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
  *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.bootstrap.test.common;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Properties;

import org.jboss.bootstrap.BaseServerConfig;
import org.jboss.bootstrap.spi.ServerConfig;

/**
 * ServerConfigUtil
 * 
 * Helper methods for dealing with ServerConfig objects
 *
 * @author <a href="mailto:andrew.rubinger@jboss.org">ALR</a>
 * @version $Revision: $
 */
public class ServerConfigUtil
{

   /**
    * Non-instanciable
    */
   private ServerConfigUtil()
   {

   }

   //--------------------------------------------------------------------||
   // Utilities ---------------------------------------------------------||
   //--------------------------------------------------------------------||

   /**
    * Obtains a JBOSS_HOME file using the current location of the code source.
    * The actual home will be in the directory "jbossHome" of the current location 
    */
   public static final File getJBossHomeFromCurrentLocation()
   {
      return getJBossHomeFromCurrentLocation("jbossHome");
   }

   /**
    * Obtains a JBOSS_HOME file using the current location of the code source.
    * The actual home will be in the specified subdirectory of the current location 
    */
   public static final File getJBossHomeFromCurrentLocation(String subdirectoryOfCurrent)
   {
      // Ensure we've been given a 
      assert subdirectoryOfCurrent != null : "Specified JBOSS_HOME child dir from current location is unspecified";

      // Obtain current URLs and build a JBOSS_HOME base
      URL currentUrl = ServerConfigUtil.class.getProtectionDomain().getCodeSource().getLocation();
      File currentFile = null;
      try
      {
         currentFile = new File(currentUrl.toURI());
      }
      catch (URISyntaxException e)
      {
         throw new RuntimeException(e);
      }
      File homeFile = new File(currentFile, subdirectoryOfCurrent);
      return homeFile;
   }

   /**
    * Obtains a ServerConfig from the specified properties.  Will additionally:
    * 
    * - Add the ServerConfig.HOME_DIR property is not specified
    * - Make the directories required for ServerConfig.HOME_DIR 
    */
   public static final BaseServerConfig getConfig(Properties props)
   {
      // Define a JBOSS_HOME and default server
      File homeFile = getJBossHomeFromCurrentLocation();
      File serverFile = new File(homeFile, "server/default");

      // Make server dir
      if (!serverFile.isDirectory() && !serverFile.mkdirs())
      {
         throw new RuntimeException("Could not create/get server directory " + serverFile);
      }

      // Add ServerConfig.HOME_DIR if not specified
      if (props.getProperty(ServerConfig.HOME_DIR) == null)
      {
         try
         {
            props.setProperty(ServerConfig.HOME_DIR, homeFile.toURL().toString());
         }
         catch (MalformedURLException e)
         {
            throw new RuntimeException(e);
         }
      }

      // Create the config
      BaseServerConfig config = null;
      try
      {
         config = new BaseServerConfig(props);
      }
      catch (Exception e)
      {
         throw new RuntimeException("Could not create new server config", e);
      }

      // Return
      return config;
   }

}
